@@ -1,12 +1,12 @@
 #!/usr/bin/env python3
-
 import numpy as np
 import matplotlib
-matplotlib.use('Agg')
 try:
     import CIAlign.cropSeq as cropSeq
 except ImportError:
     import cropSeq
+matplotlib.use('Agg')
+
 
 def cropEnds(arr, nams, relativePositions, rmfile, log, mingap, redefine_perc):
     '''
@@ -51,26 +51,30 @@ def cropEnds(arr, nams, relativePositions, rmfile, log, mingap, redefine_perc):
             nam = nams[i]
             non_gap_start = sum(row[0:start] != "-")
             non_gap_end = sum(row[end:] != "-")
+            # list of positions between 0 and start which are not gaps
+            startpos = np.where(row[0:start] != "-")[0]
+            # list of positions from end to end of the sequences
+            # which are not gaps
+            endpos = np.where(row[end:] != "-")[0] + end
+
+            rel_startpos = np.array(relativePositions)[startpos]
+            rel_endpos = np.array(relativePositions)[endpos]
+
             if non_gap_start != 0:
                 log.info("Removed %i bases from start of %s" % (
                         non_gap_start, nam))
-                out.write("Removed %i bases from start of %s" % (
-                        non_gap_start, nam))
-                out.write('\n')
+                startpos_str = [str(x) for x in rel_startpos]
+                out.write("crop_ends\t%s\t%s\n" % (nam,
+                                                   ",".join(startpos_str)))
             if non_gap_end != 0:
                 log.info("Removed %i bases from end of %s" % (
                         non_gap_end, nam))
-                out.write("Removed %i bases from end of %s" % (
-                        non_gap_end, nam))
-                out.write('\n')
-            # list of positions between 0 and start which are not gaps
-            startpos = np.where(row[0:start] != "-")[0]
-            # list of positions from end to end of the sequences which are not gaps
-            endpos = np.where(row[end:] != "-")[0] + end
-            rel_startpos = np.array(relativePositions)[startpos]
-            rel_endpos = np.array(relativePositions)[endpos]
+                endpos_str = [str(x) for x in rel_endpos]
+                out.write("crop_ends\t%s\t%s\n" % (nam,
+                                                   ",".join(endpos_str)))
+
             r[nam] = ((rel_startpos, rel_endpos))
-            #r[nam] = ((startpos, endpos))
+            # r[nam] = ((startpos, endpos))
         newarr.append(list(newseq))
     out.close()
     return (np.array(newarr), r)
@@ -101,6 +105,9 @@ def removeDivergent(arr, nams, rmfile, log, percidentity=0.75):
     r: set
         A set of names of sequences which have been removed
     '''
+    out = open(rmfile, "a")
+    if out.closed:
+        print('file is closed')
     j = 0
     keep = []
     for a in arr:
@@ -131,9 +138,10 @@ def removeDivergent(arr, nams, rmfile, log, percidentity=0.75):
     keep = np.array(keep)
     newarr = arr[keep, :]
     r = set(np.array(nams)[np.invert(keep)])
-    log.info("Removing divergent sequences %s" % (", ".join(list(r))))
-
-
+    if len(r) != 0:
+        log.info("Removing divergent sequences %s" % (", ".join(list(r))))
+        out.write("remove_divergent\t%s\n" % (",".join(list(r))))
+    out.close()
     return (newarr, r)
 
 
@@ -171,6 +179,8 @@ def removeInsertions(arr, relativePositions, rmfile, log,
     '''
     log.info("Removing insertions\n")
     out = open(rmfile, "a")
+    if out.closed:
+        print('file is closed')
     # record which sites are not "-"
     boolarr = arr != "-"
     # array of the number of non-gap sites in each column
@@ -178,58 +188,65 @@ def removeInsertions(arr, relativePositions, rmfile, log,
     # run a sliding window along the alignment and check for regions
     # which have higher coverage at the ends of the window than in the
     # middle - store these in put_indels
-    put_indels = set()
+    put_indels = []
     for size in range(min_size, max_size, 1):
         for i in range(0, len(sums)+1 - size, 1):
             these_sums = sums[i:i+size]
             # take the number of non gap positions
             # for each column in this window
-            ns = np.array(range(i, i+size))
+            ns = np.arange(i, i+size)
             left = these_sums[0]
             right = these_sums[-1]
             # record sites with lower coverage than their flanking seqs
             # and less than mincov total coverage
             x = set(ns[(these_sums < left) &
                        (these_sums < right)])
-            put_indels = put_indels | x
+            if len(x) >= min_size:
+                put_indels += list(x)
+    put_indels = set(put_indels)
     put_indels = np.array(sorted(list(put_indels)))
     # for the putative indels, check if there are more sequences
     # with a gap at this position (but with sequence on either side)
     # than with no gap (but with sequence on either side)
     rmpos = set()
     absolutePositions = set()
+    i = 0
+
     for p in put_indels:
-        left = arr[:, :p]
-        right = arr[:, p+1:]
-        pcol = arr[:, p]
-        pcol_nongaps = pcol != "-"
-        pcol_gaps = pcol == "-"
-        leftsum = sum(left.T != "-")
-        rightsum = sum(right.T != "-")
+        left = boolarr[:, :p]
+        right = boolarr[:, p+1:]
+        pcol = boolarr[:, p]
+        pcol_nongaps = pcol
+        pcol_gaps = np.invert(pcol)
+        leftsum = np.sum(left, 1)
+        rightsum = np.sum(right, 1)
         covers_region = (sum((pcol_nongaps) & (
                 leftsum >= min_flank) & (rightsum >= min_flank)))
         lacks_region = (sum((pcol_gaps) & (
                 leftsum >= min_flank) & (rightsum >= min_flank)))
         if lacks_region > covers_region:
             absolutePositions.add(p)
+        i += 1
+
     # make a list of positions to remove
     rm_relative = set()
     for n in absolutePositions:
         rm_relative.add(relativePositions[n])
     for n in rm_relative:
         relativePositions.remove(n)
-    #for n in absolutePositions:
-    #    relativePositions.remove(n)
+    # for n in absolutePositions:
+    #     relativePositions.remove(n)
     rmpos = np.array(list(absolutePositions))
 
     keeppos = np.arange(0, len(sums))
     keeppos = np.invert(np.in1d(keeppos, rmpos))
-    log.info("Removing sites %s" % (", ".join([str(x) for x in rmpos])))
-    out.write("Removing sites %s" % (", ".join([str(x) for x in rmpos])))
-    out.write('\n')
+    if len(rmpos) != 0:
+        rmpos_str = [str(x) for x in rm_relative]
+        log.info("Removing sites %s" % (", ".join(rmpos_str)))
+        out.write("remove_insertions\t%s\n" % (",".join(rmpos_str)))
     out.close()
     arr = arr[:, keeppos]
-    #return (arr, set(rmpos), relativePositions)
+    # return (arr, set(rmpos), relativePositions)
     return (arr, set(rm_relative), relativePositions)
 
 
@@ -259,15 +276,21 @@ def removeTooShort(arr, nams, rmfile, log, min_length):
          A set of names of sequences which have been removed
 
     '''
+    out = open(rmfile, "a")
+    if out.closed:
+        print('file is closed')
     if len(arr) != 0:
         arrT = arr.transpose()
         sums = sum(arrT != "-")
         arr = arr[sums > min_length]
         rmnames = set(np.array(nams)[sums <= min_length])
-        log.info("Removing sequences %s" % (", ".join(list(rmnames))))
+        if len(rmnames) != 0:
+            log.info("Removing too short sequences %s" % (
+                ", ".join(list(rmnames))))
+            out.write("remove_too_short\t%s\n" % ",".join(list(rmnames)))
     else:
         rmnames = set()
-
+    out.close()
     return (arr, rmnames)
 
 
@@ -312,15 +335,13 @@ def removeGapOnly(arr, relativePositions, rmfile, log):
             relativePositions.remove(n)
         rmpos = set(rmpos)
         arr = arr[:, sums != len(arr[:, 0])]
-        log.info("Removing gap only sites %s" % (
-                ", ".join([str(x) for x in rmpos])))
-        out.write("Removing gap only sites %s" % (
-                ", ".join([str(x) for x in rmpos])))
-        out.write('\n')
-
+        if len(rmpos) != 0:
+            rmpos_str = [str(x) for x in rmpos]
+            log.info("Removing gap only sites %s" % (
+                    ", ".join(rmpos_str)))
+            out.write("remove_gap_only\t%s\n" % (",".join(rmpos_str)))
     else:
         rmpos = set()
 
-
     out.close()
     return (arr, rmpos, relativePositions)
